/* DoctorMX Library for Mac OSX */
/* (c)2010 Yuji Kuwabara  kuwatec, Inc. */

#ifndef	_DDrMXLibX_h_
#define	_DDrMXLibX_h_


#ifdef	__cplusplus
extern "C" {
#endif


typedef	void*	RDrMXEnum;	/* reference to enumerator */
typedef	void*	RDrMX;		/* reference to device */


/* device enumerator */

/*
	returns NULL if failed to create enumerator, otherwise valid enumerator reference.
*/
extern	RDrMXEnum	DrMXEnum_New();

/*
	aR   : enumerator reference returned by DrMXEnum_New().
*/
extern	void		DrMXEnum_Delete(RDrMXEnum aE);

/*
	aR   : enumerator reference returned by DrMXEnum_New().

	returns device ID (serial number if available, otherwise location id). NULL if no more.
*/
extern	CFStringRef	DrMXEnum_Next(RDrMXEnum aE);


/* device */

/* opens first available device
	aRL   : run loop on which completion callback will be executed.

	returns NULL if failed to open, otherwise valid device reference.
*/
extern	RDrMX	DrMX_Open(CFRunLoopRef aRL);

/* opens specified device
	aRL   : run loop on which completion callback will be executed.
	aID   : device id returned by DrMXEnum_Next().

	returns NULL if failed to open, otherwise valid device reference.
*/
extern	RDrMX	DrMX_OpenID(CFRunLoopRef aRL, CFStringRef aID);

/*
	aR   : device reference returned by DrMX_Open().
*/
extern	void	DrMX_Close(RDrMX aR);

/*
	aR   : device reference returned by DrMX_Open().
	aP 	 : pointer to a byte array. 1st byte is a start code.
	aL   : length (513 byte max. greater than 0)
	aCB  : completion callback function
	aRef : arbitrary reference constant passed to aCB

	returns 1 if accepted (pending), otherwise 0.
*/
extern	int		DrMX_Write(RDrMX aR, const unsigned char* aP, int aL, void(*aCB)(void*), void* aRef);

/*
	aR   : device reference returned by DrMX_Open().
	aP   : pointer to a byte array. 1st byte will be a start code.
	aL   : maximum length (513 byte max)
	aCB  : completion callback function. actual read byte count will be passed.
	aRef : arbitrary reference constant passed to aCB

	returns -1 if accepted (pending), 
			0 < if read immediately (aCB is not called back),
			0 if pending or error
*/
extern	int		DrMX_Read(RDrMX aR, unsigned char* aP, int aL, void(*aCB)(void*, int), void* aRef);

/*
  aBrk : break space extension, in addition to minimum 92, default 84 (176uS) [0 255] [uS].
  aMAB : Mark After Break extension, in addition to minimum 12, default 0 [0 255] [uS].
  aMBS : Mark Between Slot, default 0 [0 255] [uS].
 returns 1 if succeeded, (0 if failed)
*/
extern	int		DrMX_SetTiming(RDrMX aR, unsigned int aBrk, unsigned int aMAB, unsigned int aMBS);


#ifdef	__cplusplus
}
#endif


#endif
