// DoctorMX Library Test //

#include	<stdio.h>
#include	<Carbon/Carbon.h>
#include	"DDrMXLibX.h"


struct	STest
{
	int				state;
	int				readc;
	RDrMX			drmx;
	CFRunLoopRef	rlref;
	unsigned char	wb[513];
	unsigned char	rb[513];
};


static	void	WCB(void* aRef)
{
	STest*	p = (STest*)aRef;

	puts("DrMX_Write completed");

	p->state = 1;
}


static	void	RCB(void* aRef, int aL)
{
	STest*	p = (STest*)aRef;

	puts("DrMX_Read completed");

	if (p->state == 0)	// again
	{
		int	r = DrMX_Read(p->drmx, p->rb, 513, RCB, aRef);
	}
	else
	{
		p->readc = aL;
		p->state = 2;
	}
}


static	OSStatus	IoTask(void* aRef)
{
	STest*					p = (STest*)aRef;
	CFRunLoopSourceContext	ctx;

	memset(&ctx, 0, sizeof(ctx));

	CFRunLoopSourceRef	rls = CFRunLoopSourceCreate(NULL, 0, &ctx);
	CFRunLoopRef		rl = CFRunLoopGetCurrent();

	CFRunLoopAddSource(rl, rls, kCFRunLoopCommonModes);
	p->rlref = rl;
	CFRunLoopRun();
	return 0;
}


static	void	PrintStr(CFStringRef aS)
{
	char	sb[0x40];
	Boolean	r = CFStringGetCString(aS, sb, sizeof(sb), kCFStringEncodingMacRoman);
	
	if (r)
	{
		printf("%s", sb);
	}
}


class	CTimer
{
	EventTime	base;

public:
	CTimer() : base(GetCurrentEventTime()) {}
	bool	expire(int aSec) const
	{
		if (aSec <= (GetCurrentEventTime() - base))
		{
			puts("Time out.");
			return TRUE;
		}
		return FALSE;
	}
};


int		main()
{
	puts("DrMXLib Test");

	CFStringRef	id = NULL;	// last enumerated id

	{
		RDrMXEnum	de = DrMXEnum_New();

		if (de != NULL)
		{
			puts("Enumerate DrMX device.");
		
			for (;;)
			{
				CFStringRef	s = DrMXEnum_Next(de);

				if (s == NULL) break;
				PrintStr(s);
				puts("");
				if (id != NULL) CFRelease(id);
				id = s;
			}
			puts("");
			DrMXEnum_Delete(de);
		}
	}

	STest		test;
	MPTaskID	tid;

	test.state = 0;
	test.readc = 0;		
	test.rlref = NULL;		

	MPCreateTask(IoTask, &test, 0, 0, 0, 0, 0, &tid);
	{
		CTimer	t;

		while (test.rlref == NULL && !t.expire(1)) {MPYield();}
	}

	test.drmx = DrMX_OpenID(test.rlref, id);

	if (test.drmx != NULL)
	{
		printf("DrMX is opened with ");
		PrintStr(id);
		printf(".\n");

		int		r;
		
		memset(test.wb, 0, sizeof(test.wb));
		memset(test.rb, 1, sizeof(test.rb));

		for (int i = 0; i < 513; ++i)
		{
			test.wb[i] = i;
		}
		
		r = DrMX_Read(test.drmx, test.rb, 513, RCB, &test);
		if (0 < r)			// immediate (may be residue, try again)
		{
			RCB(&test, r);
		}
		else if (r == 0)	// error
		{
			puts("Failed to read");
		}

		r = DrMX_Write(test.drmx, test.wb, 513, WCB, &test);
		if (r)	// succeeded
		{	// wait for write completion
			CTimer	t;

			while (test.state < 1 && !t.expire(1)) {MPYield();}
		}
		else	// error
		{
			puts("Cannot write to DrMX.");
		}
		r = DrMX_Write(test.drmx, test.wb, 513, NULL, NULL);	// closure

		{	// wait for read completion
			CTimer	t;

			while (test.state < 2 && !t.expire(1)) {MPYield();}
		}

		if (test.readc != 513)	// unexpected read length
		{	// may be incomplete residue, try read again
			puts("Read again.");

			test.state = 1;
			test.readc = 0;
			r = DrMX_Read(test.drmx, test.rb, 513, RCB, &test);
			if (0 < r)			// immediate
			{
				test.readc = r;
			}
			else if (r == 0)	// error
			{
				puts("Failed to read");
			}
			else
			{	// wait for read completion
				CTimer	t;

				while (test.state < 2 && !t.expire(1)) {MPYield();}
			}
		}

		if (test.readc == 513)
		{
			if (memcmp(test.wb, test.rb, 513) == 0)
			{
				puts("Write/Read match.");
			}
			else
			{
				puts("Write/Read unmatch.");
			}
		}
		else
		{
			printf("Cannot read from DrMX. (%d)\n", test.readc);
		}

		DrMX_Close(test.drmx);
	}
	else
	{
		puts("DrMX is not opened.");
	}

	if (id != NULL) CFRelease(id);

	return 0;
}
